@@ -1,48 +1,55 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 
+import colorsys
 import gettext
+import locale
+import logging
 import os
 import platform
+import re
 import subprocess
 import tempfile
-import re
-import logging
 import unicodedata
-import colorsys
 import xml.etree.ElementTree as ET
 from collections import defaultdict
 from decimal import Decimal
+from http.server import BaseHTTPRequestHandler, HTTPServer
+from pathlib import PurePath
+from urllib.parse import parse_qs, urlencode, urlparse, urlunparse
+
 try:
     from http import HTTPStatus
 except ImportError:
     from http import client as HTTPStatus
-from functools import wraps, lru_cache
-from tryton.config import CONFIG
-from tryton.config import TRYTON_ICON, PIXMAPS_DIR
-import sys
-import webbrowser
-import traceback
-import tryton.rpc as rpc
-import socket
-import _thread
-import urllib.request
-import urllib.parse
-import urllib.error
-from string import Template
+
 import shlex
+import socket
+import sys
+import traceback
+import urllib.error
+import urllib.parse
+import urllib.request
+import webbrowser
+from functools import lru_cache, wraps
+from string import Template
+from threading import Lock, Thread
+
+import tryton.rpc as rpc
+from tryton.config import CONFIG, PIXMAPS_DIR, SOUNDS_DIR, TRYTON_ICON
+
 try:
     import ssl
 except ImportError:
     ssl = None
 import zipfile
-from threading import Lock
 
-from gi.repository import Gdk, GdkPixbuf, GLib, GObject, Gtk
+from gi.repository import Gdk, GdkPixbuf, Gio, GLib, GObject, Gtk
 
 from tryton import __version__
-from tryton.exceptions import TrytonServerError, TrytonError
+from tryton.exceptions import TrytonError, TrytonServerError
 from tryton.pyson import PYSONEncoder
+
 from .underline import set_underline
 from .widget_style import widget_class
 
@@ -96,6 +103,8 @@ class IconFactory:
         try:
             icon_ref = (cls._name2id[iconname], iconname)
         except KeyError:
+            logger.error(f"Unknown icon {iconname}")
+            cls._icons[iconname] = None
             return
         idx = cls._tryton_icons.index(icon_ref)
         to_load = slice(max(0, idx - cls.batchnum // 2),
@@ -120,13 +129,14 @@ class IconFactory:
         colors = CONFIG['icon.colors'].split(',')
         cls.register_icon(iconname)
         if iconname not in cls._pixbufs[(size, badge)]:
+            data = None
             if iconname in cls._icons:
                 data = cls._icons[iconname]
             elif iconname in cls._local_icons:
                 path = cls._local_icons[iconname]
                 with open(path, 'rb') as fp:
                     data = fp.read()
-            else:
+            if not data:
                 logger.error("Unknown icon %s" % iconname)
                 return
             if not color:
@@ -288,6 +298,25 @@ class ModelName:
 MODELNAME = ModelName()
 
 
+class ModelNotification:
+    _depends = None
+
+    def load_names(self):
+        try:
+            self._depends = rpc.execute(
+                'model', 'ir.model', 'get_notification', rpc.CONTEXT)
+        except TrytonServerError:
+            pass
+
+    def get(self, model):
+        if self._depends is None:
+            self.load_names()
+        return self._depends.get(model, [])
+
+
+MODELNOTIFICATION = ModelNotification()
+
+
 class ViewSearch(object):
     searches = {}
 
@@ -301,9 +330,8 @@ class ViewSearch(object):
 
     def load_searches(self):
         try:
-            self.searches = rpc.execute('model', 'ir.ui.view_search',
-                'get_search', rpc.CONTEXT)
-        except TrytonServerError:
+            self.searches = RPCExecute('model', 'ir.ui.view_search', 'get')
+        except RPCException:
             self.searches = {}
 
     def __getitem__(self, model):
@@ -311,19 +339,16 @@ class ViewSearch(object):
 
     def add(self, model, name, domain):
         try:
-            id_, = RPCExecute('model', 'ir.ui.view_search',
-                'create', [{
-                        'model': model,
-                        'name': name,
-                        'domain': self.encoder.encode(domain),
-                        }])
+            id_ = RPCExecute(
+                'model', 'ir.ui.view_search', 'set',
+                name, model, self.encoder.encode(domain))
         except RPCException:
             return
         self.searches.setdefault(model, []).append((id_, name, domain, True))
 
     def remove(self, model, id_):
         try:
-            RPCExecute('model', 'ir.ui.view_search', 'delete', [id_])
+            RPCExecute('model', 'ir.ui.view_search', 'unset', id_)
         except RPCException:
             return
         for i, domain in enumerate(self.searches[model]):
@@ -419,16 +444,8 @@ def file_selection(title, filename='',
         action=Gtk.FileChooserAction.OPEN, preview=True, multi=False,
         filters=None):
     parent = get_toplevel_window()
-    if action == Gtk.FileChooserAction.OPEN:
-        buttons = (set_underline(_("Cancel")), Gtk.ResponseType.CANCEL,
-            set_underline(_("Select")), Gtk.ResponseType.OK)
-    else:
-        buttons = (set_underline(_("Cancel")), Gtk.ResponseType.CANCEL,
-            set_underline(_("Save")), Gtk.ResponseType.OK)
-    win = Gtk.FileChooserDialog(
+    win = Gtk.FileChooserNative(
         title=title, transient_for=parent, action=action)
-    win.add_buttons(*buttons)
-    win.set_icon(TRYTON_ICON)
     if filename:
         if action in (Gtk.FileChooserAction.SAVE,
                 Gtk.FileChooserAction.CREATE_FOLDER):
@@ -439,7 +456,6 @@ def file_selection(title, filename='',
     if hasattr(win, 'set_do_overwrite_confirmation'):
         win.set_do_overwrite_confirmation(True)
     win.set_select_multiple(multi)
-    win.set_default_response(Gtk.ResponseType.OK)
     if filters is not None:
         for filt in filters:
             win.add_filter(filt)
@@ -464,12 +480,12 @@ def file_selection(title, filename='',
         win.connect('update-preview', update_preview_cb, img_preview)
 
     button = win.run()
-    if button != Gtk.ResponseType.OK:
+    if button != Gtk.ResponseType.ACCEPT:
         result = None
     elif not multi:
-        result = win.get_filename()
+        result = PurePath(win.get_filename())
     else:
-        result = win.get_filenames()
+        result = [PurePath(f) for f in win.get_filenames()]
     parent.present()
     win.destroy()
     return result
@@ -526,10 +542,8 @@ def file_open(filename, type=None, print_p=False):
                     file_open(zfilename, type=ztype, print_p=True)
         return
 
-    if os.name == 'nt':
-        operation = 'open'
-        if print_p:
-            operation = 'print'
+    if hasattr(os, 'startfile'):
+        operation = 'print' if print_p else 'open'
         try:
             os.startfile(os.path.normpath(filename), operation)
         except WindowsError:
@@ -554,12 +568,37 @@ def file_open(filename, type=None, print_p=False):
         except OSError:
             save()
     else:
+        uri = GLib.filename_to_uri(filename)
         try:
-            subprocess.Popen(['xdg-open', filename])
-        except OSError:
+            Gio.AppInfo.launch_default_for_uri(uri)
+        except GLib.Error:
             save()
 
 
+def webbrowser_open(url):
+    try:
+        Gio.AppInfo.launch_default_for_uri(url)
+    except GLib.Error:
+        webbrowser.open(url)
+
+
+def url_open(uri):
+    try:
+        return urllib.request.urlopen(uri)
+    except urllib.error.URLError:
+        if sys.platform == 'win32' and uri.startswith('file://'):
+            # There are two ways that Windows UNC filenames can be represented:
+            # file://server/folder/data.xml
+            # file:////server/folder/data.xml
+            if uri.startswith('file:////'):
+                uri = uri[len('file://'):]
+            else:
+                uri = uri[len('file:')]
+            return open(uri)
+        else:
+            raise
+
+
 def mailto(to=None, cc=None, subject=None, body=None, attachment=None):
     if CONFIG['client.email']:
         cmd = Template(CONFIG['client.email']).substitute(
@@ -603,7 +642,7 @@ def mailto(to=None, cc=None, subject=None, body=None, attachment=None):
         url += "&body=" + urllib.parse.quote(body, "")
     if attachment:
         url += "&attachment=" + urllib.parse.quote(attachment, "")
-    webbrowser.open(url, new=1)
+    webbrowser_open(url, new=1)
 
 
 class UniqueDialog(object):
@@ -845,7 +884,7 @@ class ErrorDialog(UniqueDialog):
             CONFIG['bug.url'], _("Report Bug"))
         button_roundup.get_child().set_halign(Gtk.Align.START)
         button_roundup.connect('activate-link',
-            lambda widget: webbrowser.open(CONFIG['bug.url'], new=2))
+            lambda widget: webbrowser_open(CONFIG['bug.url'], new=2))
         dialog.vbox.pack_start(
             button_roundup, expand=False, fill=False, padding=0)
 
@@ -855,8 +894,7 @@ class ErrorDialog(UniqueDialog):
         if isinstance(title, Exception):
             title = "%s: %s" % (title.__class__.__name__, title)
         details += '\n' + title
-        log = logging.getLogger(__name__)
-        log.error(details)
+        logger.error(details)
         return super(ErrorDialog, self).__call__(title, details)
 
 
@@ -866,7 +904,7 @@ error = ErrorDialog()
 def check_version(box, version=__version__):
     def info_bar_response(info_bar, response, box, url):
         if response == Gtk.ResponseType.ACCEPT:
-            webbrowser.open(url)
+            webbrowser_open(url)
         box.remove(info_bar)
 
     class HeadRequest(urllib.request.Request):
@@ -912,6 +950,18 @@ def check_version(box, version=__version__):
         return False
 
 
+def open_documentation():
+    version = __version__.split('.')[:2]
+    if int(version[-1]) % 2:
+        version = 'latest'
+    else:
+        version = '.'.join(version)
+    webbrowser_open(CONFIG['doc.url'] % {
+            'lang': CONFIG['client.lang'],
+            'version': version,
+            })
+
+
 def to_xml(string):
     return string.replace('&', '&amp;'
         ).replace('<', '&lt;').replace('>', '&gt;')
@@ -930,11 +980,9 @@ def process_exception(exception, *args, **kwargs):
             name, msg, description = exception.args
             res = userwarning(description, msg)
             if res in ('always', 'ok'):
-                RPCExecute('model', 'res.user.warning', 'create', [{
-                            'user': rpc._USER,
-                            'name': name,
-                            'always': (res == 'always'),
-                            }],
+                RPCExecute(
+                    'model', 'res.user.warning', 'skip',
+                    name, (res == 'always'),
                     process_exception=False)
                 return rpc_execute(*args)
         elif exception.faultCode == 'UserError':
@@ -961,12 +1009,14 @@ def process_exception(exception, *args, **kwargs):
             from tryton.gui.main import Main
             if PLOCK.acquire(False):
                 try:
-                    Login()
+                    get_credentials(rpc._USER)
                 except TrytonError as exception:
-                    if exception.faultCode == 'QueryCanceled':
-                        Main().on_quit()
-                        sys.exit()
-                    raise
+                    if exception.faultCode != 'QueryCanceled':
+                        message(
+                            _("Could not get a session."),
+                            msg_type=Gtk.MessageType.ERROR)
+                    Main().on_quit()
+                    sys.exit()
                 finally:
                     PLOCK.release()
                 if args:
@@ -975,6 +1025,8 @@ def process_exception(exception, *args, **kwargs):
             message(
                 _('Too many requests. Try again later.'),
                 msg_type=Gtk.MessageType.ERROR)
+        elif exception.faultCode == str(int(HTTPStatus.NOT_FOUND)):
+            message(_("Not found."), msg_type=Gtk.MessageType.ERROR)
         else:
             error(exception, exception.faultString)
     else:
@@ -982,6 +1034,60 @@ def process_exception(exception, *args, **kwargs):
     raise RPCException(exception)
 
 
+def get_credentials(user_id=None):
+    if not CONFIG['login.service']:
+        Login()
+    else:
+        url = CONFIG['login.service']
+        port = CONFIG['login.service.port']
+        next_ = 'http://localhost:%s/' % port
+        url_parts = list(urlparse(url))
+        query = dict(parse_qs(url_parts[4]))
+        query['next'] = next_
+        if user_id is not None:
+            query['renew'] = user_id
+        url_parts[4] = urlencode(query)
+        url = urlunparse(url_parts)
+        webbrowser_open(url)
+
+        class RequestHandler(BaseHTTPRequestHandler):
+            def do_GET(self):
+                self.send_response(200)
+                self.send_header('Content-Type', 'text/html')
+                self.end_headers()
+                try:
+                    rpc.set_service_session(
+                        parse_qs(urlparse(self.path).query))
+                except ValueError:
+                    pass
+                else:
+                    self.server.session_set = True
+                self.wfile.write(b"""\
+<html>
+    <head>
+        <title>Authentication Status</title>
+        <script>
+        window.onload = function() {
+            window.close();
+        }
+        </script>
+    </head>
+    <body>
+        <p>The authentication flow has completed.</p>
+    </body>
+</html>""")
+
+            def log_message(self, *args):
+                pass
+
+        server = HTTPServer(('localhost', port), RequestHandler)
+        server.timeout = 5 * 60
+        server.session_set = False
+        server.handle_request()
+        if not server.session_set:
+            raise TrytonError('SessionFailed')
+
+
 class Login(object):
     def __init__(self, func=rpc.login):
         parameters = {}
@@ -998,6 +1104,10 @@ class Login(object):
                         _('Too many requests. Try again later.'),
                         msg_type=Gtk.MessageType.ERROR)
                     continue
+                elif (exception.faultCode
+                        == str(int(HTTPStatus.NOT_FOUND))):
+                    message(_("Not Found."), msg_type=Gtk.MessageType.ERROR)
+                    raise TrytonError('QueryCanceled')
                 if exception.faultCode != 'LoginException':
                     raise
                 name, msg, type = exception.args
@@ -1090,6 +1200,7 @@ class RPCProgress(object):
         self.res = None
         self.error = False
         self.exception = None
+        self._cursor_timeout = None
 
     def start(self):
         try:
@@ -1101,7 +1212,7 @@ class RPCProgress(object):
         else:
             if not self.res:
                 self.error = True
-        if self.callback:
+        if self.callback and CONFIG['thread']:
             # Post to GTK queue to be run by the main thread
             GLib.idle_add(self.process)
         return True
@@ -1110,23 +1221,31 @@ class RPCProgress(object):
         self.process_exception_p = process_exception_p
         self.callback = callback
 
-        if callback:
+        if callback and CONFIG['thread']:
             # Parent is only useful if it is asynchronous
             # otherwise the cursor is not updated.
             self.parent = get_toplevel_window()
+            self._cursor_timeout = GLib.timeout_add(3000, self._set_cursor)
+            Thread(target=self.start).start()
+            return
+        else:
+            self.start()
+            return self.process()
+
+    def _set_cursor(self):
+        self._cursor_timeout = None
+        if self.parent:
             window = self.parent.get_window()
             if window:
                 display = window.get_display()
                 watch = Gdk.Cursor.new_for_display(
                     display, Gdk.CursorType.WATCH)
                 window.set_cursor(watch)
-            _thread.start_new_thread(self.start, ())
-            return
-        else:
-            self.start()
-            return self.process()
 
     def process(self):
+        if self._cursor_timeout:
+            GLib.source_remove(self._cursor_timeout)
+            self._cursor_timeout = None
         if self.parent and self.parent.get_window():
             self.parent.get_window().set_cursor(None)
 
@@ -1246,11 +1365,30 @@ def untimezoned_date(date):
     return timezoned_date(date, reverse=True)
 
 
-def humanize(size):
-    for x in ('bytes', 'KB', 'MB', 'GB', 'TB', 'PB'):
-        if size < 1000:
-            return '%3.1f%s' % (size, x)
-        size /= 1000.0
+def humanize(size, suffix=''):
+    if 0 < abs(size) < 1:
+        for u in ['', 'm', 'µ', 'n', 'p', 'f', 'a', 'z', 'y', 'r', 'q']:
+            if abs(size) >= 0.01:
+                break
+            size *= 1000.0
+        else:
+            size /= 1000.0
+    else:
+        for u in ['', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'R', 'Q']:
+            if abs(size) <= 1000:
+                break
+            size /= 1000.0
+        else:
+            size *= 1000.0
+    if isinstance(size, int) or size.is_integer():
+        size = locale.localize(str(int(size)))
+    elif abs(size) < 0.01:
+        size = locale.localize(
+            '{0:f}'.format(size).rstrip('0').rstrip('.'))
+    else:
+        size = locale.localize(
+            '{0:.{1}f}'.format(size, 2).rstrip('0').rstrip('.'))
+    return ''.join([size, u, suffix])
 
 
 def get_hostname(netloc):
@@ -1323,13 +1461,12 @@ def ellipsize(string, length):
 def get_align(float_, expand=True):
     "Convert float align into Gtk.Align"
     value = float(float_)
+    if expand:
+        return Gtk.Align.FILL
     if value < 0.5:
         return Gtk.Align.START
     elif value == 0.5:
-        if expand:
-            return Gtk.Align.FILL
-        else:
-            return Gtk.Align.CENTER
+        return Gtk.Align.CENTER
     else:
         return Gtk.Align.END
 
@@ -1350,6 +1487,14 @@ def setup_window(window):
         window.set_mnemonic_modifier(Gdk.ModifierType.CONTROL_MASK)
 
 
+def play_sound(self, sound='success'):
+    try:
+        from playsound import playsound
+        playsound(os.path.join(SOUNDS_DIR, f'{sound}.wav'))
+    except ImportError:
+        pass
+
+
 def get_gdk_backend():
     if sys.platform == 'darwin':
         return 'macos'
