@@ -9,7 +9,8 @@ import sys
 import threading
 import traceback
 import webbrowser
-from urllib.parse import urlparse, parse_qsl, unquote
+from functools import partial
+from urllib.parse import parse_qsl, unquote, urlparse
 
 from gi.repository import Gdk, GdkPixbuf, Gio, GLib, Gtk
 
@@ -18,9 +19,9 @@ import tryton.plugins
 import tryton.rpc as rpc
 import tryton.translate as translate
 from tryton.action import Action
-from tryton.common import RPCExecute, RPCException, RPCContextReload
-from tryton.common.cellrendererclickablepixbuf import \
-        CellRendererClickablePixbuf
+from tryton.common import RPCContextReload, RPCException, RPCExecute
+from tryton.common.cellrendererclickablepixbuf import (
+    CellRendererClickablePixbuf)
 from tryton.config import CONFIG, TRYTON_ICON, get_config_dir
 from tryton.exceptions import TrytonError, TrytonServerUnavailable
 from tryton.gui.window import Window
@@ -78,6 +79,7 @@ class Main(Gtk.Application):
                 ('mode-pda', 'client.modepda'),
                 ('save-tree-width', 'client.save_tree_width'),
                 ('save-tree-state', 'client.save_tree_state'),
+                ('code-scanner-sound', 'client.code_scanner_sound'),
                 ('spell-checking', 'client.spellcheck'),
                 ('check-version', 'client.check_version'),
                 ]:
@@ -100,6 +102,10 @@ class Main(Gtk.Application):
         action.connect('activate', lambda *a: self.edit_limit())
         self.add_action(action)
 
+        action = Gio.SimpleAction.new('documentation', None)
+        action.connect('activate', lambda *a: common.open_documentation())
+        self.add_action(action)
+
         self._shortcuts = None
         action = Gio.SimpleAction.new('shortcuts', None)
         action.connect('activate', lambda *a: self.shortcuts())
@@ -121,7 +127,7 @@ class Main(Gtk.Application):
             return
 
         self.window = Gtk.ApplicationWindow(application=self, title="Tryton")
-        self.window.set_default_size(960, 720)
+        self.window.set_default_size(1280, 960)
         self.window.maximize()
         self.window.set_position(Gtk.WindowPosition.CENTER)
         self.window.set_resizable(True)
@@ -171,9 +177,10 @@ class Main(Gtk.Application):
         Gtk.AccelMap.add_entry(
             '<tryton>/Form/Delete', Gdk.KEY_D, Gdk.ModifierType.CONTROL_MASK)
         Gtk.AccelMap.add_entry(
-            '<tryton>/Form/Next', Gdk.KEY_Page_Down, 0)
+            '<tryton>/Form/Next', Gdk.KEY_Down, Gdk.ModifierType.CONTROL_MASK)
         Gtk.AccelMap.add_entry(
-            '<tryton>/Form/Previous', Gdk.KEY_Page_Up, 0)
+            '<tryton>/Form/Previous',
+            Gdk.KEY_Up, Gdk.ModifierType.CONTROL_MASK)
         Gtk.AccelMap.add_entry(
             '<tryton>/Form/Switch View', Gdk.KEY_L,
             Gdk.ModifierType.CONTROL_MASK)
@@ -246,11 +253,14 @@ class Main(Gtk.Application):
 
         self.set_title()  # Adds username/profile while password is asked
         try:
-            common.Login()
+            common.get_credentials()
         except Exception as exception:
-            if (not isinstance(exception, TrytonError)
-                    or exception.faultCode != 'QueryCanceled'):
+            if not isinstance(exception, TrytonError):
                 common.error(exception, traceback.format_exc())
+            elif exception.faultCode == 'SessionFailed':
+                common.message(
+                    _("Could not get a session."),
+                    msg_type=Gtk.MessageType.ERROR)
             return self.quit()
         self.get_preferences()
 
@@ -298,6 +308,7 @@ class Main(Gtk.Application):
         form.append(_("Save Column Width"), 'app.save-tree-width')
         form.append(_("Save Tree State"), 'app.save-tree-state')
         form.append(_("Spell Checking"), 'app.spell-checking')
+        form.append(_("Play Sound for Code Scanner"), 'app.code-scanner-sound')
 
         section.append(_("PDA Mode"), 'app.mode-pda')
         section.append(_("Search Limit..."), 'app.search-limit')
@@ -306,6 +317,7 @@ class Main(Gtk.Application):
         menu.append_section(_("Options"), section)
 
         section = Gio.Menu.new()
+        section.append(_("Documentation..."), 'app.documentation')
         section.append(_("Keyboard Shortcuts..."), 'app.shortcuts')
         section.append(_("About..."), 'app.about')
         menu.append_section(_("Help"), section)
@@ -513,18 +525,25 @@ class Main(Gtk.Application):
         except RPCException:
             prefs = {}
 
-        threads = []
-        for target in (
-                common.IconFactory.load_icons,
-                common.MODELACCESS.load_models,
-                common.MODELHISTORY.load_history,
-                common.VIEW_SEARCH.load_searches,
-                ):
-            t = threading.Thread(target=target)
-            threads.append(t)
-            t.start()
-        for t in threads:
-            t.join()
+        targets = [
+            common.IconFactory.load_icons,
+            common.MODELACCESS.load_models,
+            common.MODELHISTORY.load_history,
+            common.MODELNOTIFICATION.load_names,
+            common.VIEW_SEARCH.load_searches,
+            ]
+        if CONFIG['thread']:
+            threads = []
+            for target in targets:
+                t = threading.Thread(target=target)
+                threads.append(t)
+                t.start()
+            for t in threads:
+                t.join()
+        else:
+            for target in targets:
+                target()
+
         if prefs and 'language_direction' in prefs:
             translate.set_language_direction(prefs['language_direction'])
             CONFIG['client.language_direction'] = \
@@ -650,6 +669,10 @@ class Main(Gtk.Application):
                                 ('End', _("Move to bottom")),
                                 ('BackSpace', _("Move to parent")),
                                 ]),
+                        (_("Edition"), [
+                                ('<Primary>c', _("Copy selected rows")),
+                                ('<Primary>v', _("Insert copied rows")),
+                                ]),
                         (_("Selection"), [
                                 ('<Ctrl>a <Ctrl>slash', _("Select all")),
                                 ('<Shift><Ctrl>a <Shift><Ctrl>slash',
@@ -836,7 +859,7 @@ class Main(Gtk.Application):
                 expand=False, fill=False, padding=0)
         name = page.name
         label = Gtk.Label(
-            label=common.ellipsize(name, 20),
+            label=common.ellipsize(name.split(' / ')[-1], 20),
             halign=Gtk.Align.START)
         self.tooltips.set_tip(label, page.name)
         self.tooltips.enable()
@@ -884,7 +907,6 @@ class Main(Gtk.Application):
             for i in range(len(self.pages)):
                 if self.pages[i].widget == page_widget:
                     page = self.pages.pop(i)
-                    page.signal_unconnect(self)
                     break
             self.notebook.remove_page(page_id)
 
@@ -945,6 +967,7 @@ class Main(Gtk.Application):
             dialog.show()
 
     def _open_url(self, url):
+        loads = partial(json.loads, object_hook=object_hook)
         urlp = urlparse(url)
         if not urlp.scheme == 'tryton':
             return
@@ -967,88 +990,72 @@ class Main(Gtk.Application):
             model, path = (path.split('/', 1) + [''])[:2]
             if not model:
                 return
-            res_id = None
-            mode = None
+            attributes = {}
             try:
-                view_ids = json.loads(params.get('views', '[]'))
+                attributes['view_ids'] = loads(params.get('views', '[]'))
                 if 'limit' in params:
-                    attributes['limit'] = json.loads(
-                        params.get('limit', 'null'))
-                name = json.loads(params.get('name', '""'))
-                search_value = json.loads(params.get('search_value', '[]'),
-                    object_hook=object_hook)
-                domain = json.loads(params.get('domain', '[]'),
-                    object_hook=object_hook)
-                context = json.loads(params.get('context', '{}'),
-                    object_hook=object_hook)
-                context_model = params.get('context_model')
-                tab_domain = json.loads(params.get('tab_domain', '[]'),
-                    object_hook=object_hook)
+                    attributes['limit'] = loads(params.get('limit', 'null'))
+                attributes['name'] = loads(params.get('name', '""'))
+                attributes['search_value'] = loads(
+                    params.get('search_value', '[]'))
+                attributes['domain'] = loads(params.get('domain', '[]'))
+                attributes['context'] = loads(params.get('context', '{}'))
+                attributes['context_model'] = params.get('context_model')
+                attributes['tab_domain'] = loads(
+                    params.get('tab_domain', '[]'))
             except ValueError:
                 return
             if path:
                 try:
-                    res_id = int(path)
+                    attributes['res_id'] = int(path)
                 except ValueError:
                     return
-                mode = ['form', 'tree']
+                attributes['mode'] = ['form', 'tree']
             try:
-                Window.create(model,
-                    view_ids=view_ids,
-                    res_id=res_id,
-                    domain=domain,
-                    context=context,
-                    context_model=context_model,
-                    mode=mode,
-                    name=name,
-                    search_value=search_value,
-                    tab_domain=tab_domain,
-                    **attributes)
+                Window.create(model, **attributes)
             except Exception:
                 # Prevent crashing the client
                 return
 
         def open_wizard(wizard):
+            attributes = {}
             if not wizard:
                 return
             try:
-                data = json.loads(params.get('data', '{}'),
-                    object_hook=object_hook)
-                direct_print = json.loads(params.get('direct_print', 'false'))
-                name = json.loads(params.get('name', '""'))
-                window = json.loads(params.get('window', 'false'))
-                context = json.loads(params.get('context', '{}'),
-                    object_hook=object_hook)
+                attributes['data'] = loads(params.get('data', '{}'))
+                attributes['direct_print'] = loads(
+                    params.get('direct_print', 'false'))
+                attributes['name'] = loads(params.get('name', '""'))
+                attributes['window'] = loads(params.get('window', 'false'))
+                attributes['context'] = loads(params.get('context', '{}'))
             except ValueError:
                 return
             try:
-                Window.create_wizard(
-                    wizard, data, direct_print=direct_print, name=name,
-                    context=context, window=window)
+                Window.create_wizard(wizard, **attributes)
             except Exception:
                 # Prevent crashing the client
                 return
 
         def open_report(report):
+            attributes = {}
             if not report:
                 return
             try:
-                data = json.loads(params.get('data'), object_hook=object_hook)
-                direct_print = json.loads(params.get('direct_print', 'false'))
-                context = json.loads(params.get('context', '{}'),
-                    object_hook=object_hook)
+                attributes['data'] = loads(params.get('data', '{}'))
+                attributes['direct_print'] = loads(
+                    params.get('direct_print', 'false'))
+                attributes['context'] = loads(params.get('context', '{}'))
             except ValueError:
                 return
             try:
-                Action.exec_report(
-                    report, data, direct_print=direct_print, context=context)
+                Action.exec_report(report, **attributes)
             except Exception:
                 # Prevent crashing the client
                 return
 
         def open_url():
             try:
-                url = json.loads(params.get('url', 'false'))
+                url = loads(params.get('url', 'false'))
             except ValueError:
                 return
             if url:
