@@ -2,21 +2,25 @@
 # this repository contains the full copyright notices and license terms.
 
 import configparser
-import os
 import gettext
-import threading
 import logging
+import os
+import re
+import shutil
+import tempfile
+import threading
 
 from gi.repository import GLib, GObject, Gtk
 
-from tryton import __version__
 import tryton.common as common
-from tryton.config import CONFIG, TRYTON_ICON, PIXMAPS_DIR, get_config_dir
 import tryton.rpc as rpc
+from tryton import __version__
 from tryton.common.underline import set_underline
+from tryton.config import CONFIG, PIXMAPS_DIR, TRYTON_ICON, get_config_dir
 
 _ = gettext.gettext
 logger = logging.getLogger(__name__)
+DEMO_HOSTNAME = re.compile(r"demo\d+\.\d+\.tryton\.org")
 
 
 class DBListEditor(object):
@@ -284,9 +288,13 @@ class DBListEditor(object):
 
     def refresh_databases(self, host, port):
         self.dbs_updated = threading.Event()
-        threading.Thread(target=self.refresh_databases_start,
-            args=(host, port)).start()
-        GLib.timeout_add(100, self.refresh_databases_end, host, port)
+        if CONFIG['thread']:
+            threading.Thread(target=self.refresh_databases_start,
+                args=(host, port)).start()
+            GLib.timeout_add(100, self.refresh_databases_end, host, port)
+        else:
+            self.refresh_databases_start(host, port)
+            self.refresh_databases_end(host, port)
 
     def refresh_databases_start(self, host, port):
         dbs = None
@@ -457,6 +465,7 @@ class DBLogin(object):
         self.entry_host = Gtk.Entry(hexpand=True)
         self.entry_host.connect_after('focus-out-event',
             self.clear_profile_combo)
+        self.entry_host.connect('focus-out-event', self.update_services)
         self.entry_host.set_activates_default(True)
         self.label_host.set_mnemonic_widget(self.entry_host)
         grid.attach(self.label_host, 0, 3, 1, 1)
@@ -481,24 +490,52 @@ class DBLogin(object):
         grid.attach(label_username, 0, 5, 1, 1)
 
         # Profile information
-        self.profile_cfg = os.path.join(get_config_dir(), 'profiles.cfg')
+        config_dir = get_config_dir()
+        self.profile_cfg = os.path.join(config_dir, 'profiles.cfg')
         self.profiles = configparser.ConfigParser()
-        if not os.path.exists(self.profile_cfg):
-            short_version = '.'.join(__version__.split('.', 2)[:2])
-            name = 'demo%s.tryton.org' % short_version
-            self.profiles.add_section(name)
-            self.profiles.set(name, 'host', name)
-            self.profiles.set(name, 'database', 'demo%s' % short_version)
-            self.profiles.set(name, 'username', 'demo')
-        else:
-            try:
-                self.profiles.read(self.profile_cfg)
-            except configparser.ParsingError:
-                logger.error("Fail to parse profiles.cfg", exc_info=True)
+        series = '.'.join(__version__.split('.', 2)[:2])
+        current_demo = f"demo{series}.tryton.org"
+        try:
+            self.profiles.read(self.profile_cfg)
+        except configparser.Error:
+            # reset self.profiles as parsing errors may leave wrong data in
+            # the parser
+            self.profiles = configparser.ConfigParser()
+            with tempfile.NamedTemporaryFile(
+                    delete=False, prefix='profiles_', suffix='.cfg',
+                    dir=config_dir) as temp_file:
+                temp_name = temp_file.name
+            shutil.copy(self.profile_cfg, temp_name)
+            logger.error(
+                f"Failed to parse {self.profiles_cfg}. "
+                f"A backup can be found at {temp_name}",
+                exc_info=True)
+        if not self.profiles.sections():
+            self.profiles.add_section(current_demo)
+            self.profiles.set(current_demo, 'host', current_demo)
+            self.profiles.set(current_demo, 'database', f'demo{series}')
+            self.profiles.set(current_demo, 'username', 'demo')
+        to_remove = []
         for section in self.profiles.sections():
+            host = self.profiles.get(section, 'host', fallback='')
+            if host and DEMO_HOSTNAME.match(host) and host != current_demo:
+                to_remove.append(section)
+                continue
             active = all(self.profiles.has_option(section, option)
                 for option in ('host', 'database'))
             self.profile_store.append([section, active])
+        for section in to_remove:
+            self.profiles.remove_section(section)
+        if to_remove and not self.profiles.has_section(current_demo):
+            self.profiles.add_section(current_demo)
+            self.profiles.set(current_demo, 'host', current_demo)
+            self.profiles.set(current_demo, 'database', f'demo{series}')
+            self.profiles.set(current_demo, 'username', 'demo')
+            self.profile_store.append([current_demo, True])
+
+        self.services = []
+        self.service_base = ''
+        self._services = {}
 
     def profile_manage(self, widget):
         def callback(profile_name):
@@ -535,6 +572,7 @@ class DBLogin(object):
             self.entry_login.set_text(username)
         else:
             self.entry_login.set_text('')
+        self.update_services()
 
     def clear_profile_combo(self, *args):
         netloc = self.entry_host.get_text()
@@ -569,6 +607,33 @@ class DBLogin(object):
         self.entry_database.props.visible = visibility
         self.label_database.props.visible = visibility
 
+    def update_services(self, *args):
+        host = self.entry_host.get_text()
+        hostname = common.get_hostname(host)
+        port = common.get_port(host)
+        key = (hostname, port)
+        for response_id, service in enumerate(self.services, 1):
+            button = self.dialog.get_widget_for_response(response_id)
+            button.get_parent().remove(button)
+        self.services = []
+        if key in self._services:
+            self.services, self.service_base = self._services[key]
+        elif hostname and port:
+            self.service_base, self.services = rpc.authentication_services(
+                hostname, port)
+            self._services[key] = (self.services, self.service_base)
+        for response_id, (name, url) in enumerate(self.services, 1):
+            button = Gtk.Button(label=name)
+            self.dialog.add_action_widget(button, response_id)
+            button.show()
+
+        # Re-add connect button to be last
+        button_connect = self.dialog.get_widget_for_response(
+            Gtk.ResponseType.OK)
+        button_connect.get_parent().remove(button_connect)
+        self.dialog.add_action_widget(button_connect, Gtk.ResponseType.OK)
+        self.dialog.set_default_response(Gtk.ResponseType.OK)
+
     def run(self):
         profile_name = CONFIG['login.profile']
         can_use_profile = self.profiles.has_section(profile_name)
@@ -598,6 +663,7 @@ class DBLogin(object):
             self.entry_database.set_text(db)
             self.entry_login.set_text(CONFIG['login.login'])
             self.clear_profile_combo()
+        self.update_services()
         self.dialog.show_all()
 
         self.entry_login.grab_focus()
@@ -609,7 +675,7 @@ class DBLogin(object):
         response, result = None, ('', '', '', '')
         while not all(result):
             response = self.dialog.run()
-            if response != Gtk.ResponseType.OK:
+            if response != Gtk.ResponseType.OK and response <= 0:
                 break
             self.clear_profile_combo()
             active_profile = self.combo_profile.get_active()
@@ -632,15 +698,22 @@ class DBLogin(object):
                         parent=self.dialog)
                 continue
             database = self.entry_database.get_text()
-            login = self.entry_login.get_text()
             CONFIG['login.profile'] = profile
             CONFIG['login.host'] = host
             CONFIG['login.db'] = database
             CONFIG['login.expanded'] = self.expander.props.expanded
-            CONFIG['login.login'] = login
-            result = (
-                hostname, port, database, self.entry_login.get_text())
+            if response == Gtk.ResponseType.OK:
+                authentication = self.entry_login.get_text()
+                CONFIG['login.login'] = authentication
+                CONFIG['login.service'] = ''
+            else:
+                authentication = self.services[response - 1][1]
+                authentication = (
+                    self.service_base + database + authentication)
+                CONFIG['login.login'] = ''
+                CONFIG['login.service'] = authentication
+            result = (hostname, port, database, authentication)
 
         self.dialog.destroy()
         self._window.destroy()
-        return response == Gtk.ResponseType.OK
+        return response == Gtk.ResponseType.OK or response > 0
