@@ -578,6 +578,7 @@ class Form(TabContent):
         name = str(position) if position else '_'
         selected = len(self.screen.selected_records)
         view_type = self.screen.current_view.view_type
+        next_view_type = self.screen.next_view_type
         has_views = self.screen.number_of_views > 1
         if selected > 1:
             name += '#%i' % selected
@@ -593,7 +594,9 @@ class Form(TabContent):
                     for b in self.screen.get_buttons())
             set_sensitive(button_id, bool(position) and can_be_sensitive)
         set_sensitive(
-            'switch', (position or view_type == 'form') and has_views)
+            'switch',
+            (position or view_type == 'form' or next_view_type != 'form')
+            and has_views)
         set_sensitive('remove', self.screen.deletable)
         set_sensitive('previous', self.screen.has_prev())
         set_sensitive('next', self.screen.has_next())
