@@ -333,7 +333,7 @@ class TreeXMLViewParser(XMLViewParser):
 
         self.view.treeview.append_column(column)
 
-        if 'optional' in attributes:
+        if 'optional' in attributes and name != self.exclude_field:
             self.view.optionals.append(column)
 
     def _parse_button(self, node, attributes):
@@ -760,6 +760,8 @@ class ViewTree(View):
             data = [[unquote(v) for v in l.split('\t')]
                 for l in text.splitlines()]
             break
+        else:
+            return
         col = self.treeview.get_cursor()[1]
         columns = [c for c in self.treeview.get_columns()
             if c.get_visible() and c.name]
@@ -785,13 +787,25 @@ class ViewTree(View):
             for col, value in zip(columns, line):
                 widget = self.get_column_widget(col)
                 if widget.get_textual_value(record) != value:
-                    widget.value_from_text(record, value)
+                    if hasattr(widget, 'search_remote'):
+                        field = record.group.fields[widget.attrs['name']]
+                        win = widget.search_remote(record, field, value)
+                        if len(win.screen.group) == 1:
+                            target, = win.screen.group
+                            field.set_client(
+                                record, (target.id, target.rec_name()))
+                        else:
+                            field.set_client(record, (None, ''))
+                        win.response(win, Gtk.ResponseType.CANCEL)
+                    else:
+                        widget.value_from_text(record, value)
                     if value and not widget.get_textual_value(record):
                         # Stop setting value if a value is correctly set
                         idx = len(group)
                         break
             if not record.validate():
                 break
+            record.save()
             idx += 1
         self.record = record
         self.screen.display(set_cursor=True)
@@ -1041,9 +1055,12 @@ class ViewTree(View):
         elif tree_sel.get_mode() == Gtk.SelectionMode.MULTIPLE:
             model, paths = tree_sel.get_selected_rows()
             if model and paths:
-                iter_ = model.get_iter(paths[0])
-                record = model.get_value(iter_, 0)
-                self.record = record
+                records = []
+                for path in paths:
+                    iter_ = model.get_iter(path)
+                    records.append(model.get_value(iter_, 0))
+                if self.record not in records:
+                    self.record = records[0]
             else:
                 self.record = None
 
@@ -1083,6 +1100,10 @@ class ViewTree(View):
     def display(self, force=False):
         self.treeview.display_counter += 1
         current_record = self.record
+        if current_record and current_record not in current_record.group:
+            # current record may have been removed by on_change calls without
+            # changing the current record of screen before the display
+            current_record = None
         if (force
                 or not self.treeview.get_model()
                 or self.group != self.treeview.get_model().group):
