@@ -2,17 +2,19 @@
 # this repository contains the full copyright notices and license terms.
 import gettext
 
-from tryton.gui.window import Window
 from tryton.common import MODELACCESS
+from tryton.gui.window import Window
 
 _ = gettext.gettext
 
 
 def translate_view(datas):
     model = datas['model']
-    Window.create(False, 'ir.translation', res_id=False,
-            domain=[('model', '=', model)],
-            mode=['tree', 'form'])
+    Window.create('ir.translation',
+        res_id=False,
+        domain=[('model', '=', model)],
+        mode=['tree', 'form'],
+        name=_('Translate view'))
 
 
 def get_plugins(model):
