@@ -1,11 +1,12 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import json
 import datetime
+import json
 from decimal import Decimal
-from dateutil.relativedelta import relativedelta
 from functools import reduce
 
+from dateutil.relativedelta import relativedelta
+
 
 class PYSON(object):
 
@@ -31,8 +32,7 @@ class PYSON(object):
             other = Bool(other)
         if (isinstance(self, And)
                 and not isinstance(self, Or)):
-            self._statements.append(other)
-            return self
+            return And(*self._statements, other)
         if self.types() != {bool}:
             return And(Bool(self), other)
         else:
@@ -45,8 +45,7 @@ class PYSON(object):
                 and other.types() != {bool}):
             other = Bool(other)
         if isinstance(self, Or):
-            self._statements.append(other)
-            return self
+            return Or(*self._statements, other)
         if self.types() != {bool}:
             return Or(Bool(self), other)
         else:
@@ -102,6 +101,8 @@ class PYSONEncoder(json.JSONEncoder):
                         ).pyson()
             else:
                 return Date(obj.year, obj.month, obj.day).pyson()
+        elif isinstance(obj, datetime.timedelta):
+            return TimeDelta(obj.days, obj.seconds, obj.microseconds)
         elif isinstance(obj, Decimal):
             return float(obj)
         return super(PYSONEncoder, self).default(obj)
@@ -161,6 +162,15 @@ class Eval(PYSON):
                     }, context.get(base) or {})
         return context.get(dct['v'], dct['d'])
 
+    @property
+    def basename(self):
+        name = self._value
+        if name.startswith('_parent_'):
+            name = name[len('_parent_'):]
+        if '.' in name:
+            name = name.split('.', 1)[0]
+        return name
+
 
 class Not(PYSON):
 
@@ -304,12 +314,14 @@ class Greater(PYSON):
             if isinstance(i, PYSON):
                 assert i.types().issubset({
                         int, float, type(None),
-                        datetime.datetime, datetime.date}), \
+                        datetime.datetime, datetime.date,
+                        datetime.timedelta}), \
                     'statement must be an integer, float, date or datetime'
             else:
                 assert isinstance(i, (
                         int, float, type(None),
-                        datetime.datetime, datetime.date)), \
+                        datetime.datetime, datetime.date,
+                        datetime.timedelta)), \
                     'statement must be an integer, float, date or datetime'
         if isinstance(equal, PYSON):
             if equal.types() != {bool}:
@@ -348,6 +360,8 @@ class Greater(PYSON):
                 elif isinstance(stmt, datetime.date):
                     time = datetime.time(0, 0)
                     stmt = datetime.datetime.combine(stmt, time).timestamp()
+                elif isinstance(stmt, datetime.timedelta):
+                    stmt = stmt.total_seconds()
                 dct[i] = float(stmt)
         return dct
 
@@ -390,16 +404,6 @@ class If(PYSON):
                 condition = Bool(condition)
         elif not isinstance(condition, bool):
             condition = bool(condition)
-        if isinstance(then_statement, PYSON):
-            then_types = then_statement.types()
-        else:
-            then_types = {type(then_statement)}
-        if isinstance(else_statement, PYSON):
-            else_types = else_statement.types()
-        else:
-            else_types = {type(else_statement)}
-        assert then_types == else_types, \
-            'then and else statements must be the same type'
         self._condition = condition
         self._then_statement = then_statement
         self._else_statement = else_statement
@@ -418,9 +422,14 @@ class If(PYSON):
 
     def types(self):
         if isinstance(self._then_statement, PYSON):
-            return self._then_statement.types()
+            types = self._then_statement.types()
         else:
-            return {type(self._then_statement)}
+            types = {type(self._then_statement)}
+        if isinstance(self._else_statement, PYSON):
+            types |= self._else_statement.types()
+        else:
+            types |= {type(self._else_statement)}
+        return types
 
     @staticmethod
     def eval(dct, context):
@@ -485,11 +494,17 @@ class In(PYSON):
             assert obj.types().issubset({dict, list}), \
                 'obj must be a dict or a list'
             if obj.types() == {dict}:
-                assert isinstance(key, str), 'key must be a string'
+                if isinstance(key, PYSON):
+                    assert key.types() == {str}, 'key must be a string'
+                else:
+                    assert isinstance(key, str), 'key must be a string'
         else:
             assert isinstance(obj, (dict, list))
             if isinstance(obj, dict):
-                assert isinstance(key, str), 'key must be a string'
+                if isinstance(key, PYSON):
+                    assert key.types() == {str}, 'key must be a string'
+                else:
+                    assert isinstance(key, str), 'key must be a string'
         self._key = key
         self._obj = obj
 
